"use strict";
/*
 * Copyright (c) 2018. Christoph Rodak  (https://reactivechart.com)
 */
Object.defineProperty(exports, "__esModule", { value: true });
var Extender = /** @class */ (function () {
    function Extender() {
        this.deep = false;
        this.missing = false;
    }
    Extender.prototype.getProperties = function (arg) {
        var args = [];
        for (var p in arg) {
            args.push(p);
        }
        return args;
    };
    Extender.prototype.extend = function (args) {
        var first = args[0];
        for (var i = 1; i < args.length; i++) {
            var ex = args[i];
            if (ex) {
                var props = this.getProperties(ex);
                for (var j = 0; j < props.length; j++) {
                    var k = props[j];
                    if (typeof ex === "object" && ex.hasOwnProperty(k)) {
                        if (this.deep) {
                            var left = first[k];
                            var right = ex[k];
                            if (left && typeof left === 'object' && right && typeof right === 'object') {
                                this.extend([left, right]);
                            }
                            else {
                                if (!this.missing || !first.hasOwnProperty(k)) {
                                    Object.defineProperty(first, k, Object.getOwnPropertyDescriptor(ex, k));
                                }
                            }
                        }
                        else {
                            if (!this.missing || !first.hasOwnProperty(k)) {
                                Object.defineProperty(first, k, Object.getOwnPropertyDescriptor(ex, k));
                            }
                        }
                    }
                }
            }
        }
        return first;
    };
    return Extender;
}());
exports.Extender = Extender;
var stdExtender = new Extender();
var deepExtender = new Extender();
deepExtender.deep = true;
var missingExtender = new Extender();
missingExtender.missing = true;
/**
 *
 * @param any
 * @returns {any}
 */
function extend() {
    var any = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        any[_i] = arguments[_i];
    }
    return stdExtender.extend(any);
}
exports.extend = extend;
exports.default = extend;
/**
 *
 * Merges the given objects into the first object using a deep merge
 */
function deep() {
    var any = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        any[_i] = arguments[_i];
    }
    return deepExtender.extend(any);
}
exports.deep = deep;
/**
 * Merges the given objects into the first object using given properties
 */
function missing() {
    var any = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        any[_i] = arguments[_i];
    }
    return missingExtender.extend(any);
}
exports.missing = missing;
function properties(p) {
    var ext = new Extender();
    ext.getProperties = function () {
        return p;
    };
    return ext;
}
exports.properties = properties;
function configure(config) {
    var ext = new Extender();
    properties(["deep", "missing"]).extend(ext, config);
    if (config.properties) {
        ext.getProperties = function () {
            return config.properties;
        };
    }
    return ext;
}
exports.configure = configure;
