"use strict";
/*
 * Copyright (c) 2018. Christoph Rodak  (https://reactivechart.com)
 */
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * Node of a list
 */
var Node = /** @class */ (function () {
    function Node(value, list) {
        this.value = value;
        this.list = list;
        this.removed = false;
        this.next = null;
        this.previous = null;
    }
    Node.prototype.remove = function () {
        if (!this.removed) {
            this.list.size--;
            this.next.previous = this.previous;
            this.previous.next = this.next;
            this.removed = true;
            return true;
        }
        return false;
    };
    Node.prototype.addBefore = function (data) {
        var node = this.list.createNode(data);
        node.next = this;
        node.previous = this.previous;
        node.removed = false;
        this.previous.next = node;
        this.previous = node;
        this.list.size++;
    };
    Node.prototype.addAfter = function (data) {
        var node = this.list.createNode(data);
        node.previous = this;
        node.next = this.next;
        node.removed = false;
        this.next.previous = node;
        this.next = node;
        this.list.size++;
    };
    return Node;
}());
/**
 * A double linked list
 * @param E the type of the list elements
 */
var List = /** @class */ (function () {
    function List() {
        this.size = 0;
        this.first = this.createNode(null);
        this.last = this.createNode(null);
        this.last.next = this.last;
        this.last.previous = this.first;
        this.first.next = this.last;
        this.first.previous = this.first;
    }
    List.prototype.createNode = function (value) {
        return new Node(value, this);
    };
    List.prototype.addLast = function (element) {
        var ln = this.createNode(element);
        this.addLastNode(ln);
        return ln;
    };
    List.prototype.addLastNode = function (node) {
        this.size++;
        node.next = this.last;
        node.previous = this.last.previous;
        node.removed = false;
        this.last.previous.next = node;
        this.last.previous = node;
    };
    List.prototype.addFirst = function (element) {
        var node = this.createNode(element);
        this.addFirstNode(node);
        return node;
    };
    List.prototype.addFirstNode = function (node) {
        this.size++;
        node.previous = this.first;
        node.next = this.first.next;
        node.removed = false;
        this.first.next.previous = node;
        this.first.next = node;
    };
    List.prototype.findAndRemove = function (element) {
        var ln = this.first;
        ln = ln.next;
        while (ln != this.last) {
            if (ln.value === element) {
                ln.remove();
                return true;
            }
            ln = ln.next;
        }
        return false;
    };
    List.prototype.contains = function (element) {
        var ln = this.first;
        ln = ln.next;
        while (ln != this.last) {
            if (ln.value === element) {
                return true;
            }
            ln = ln.next;
        }
        return false;
    };
    List.prototype.forEach = function (f) {
        var ln = this.first;
        ln = ln.next;
        while (ln != this.last) {
            f(ln.value);
            ln = ln.next;
        }
    };
    List.prototype.get = function (index) {
        var it = this.iterator();
        var i = 0;
        while (it.hasNext()) {
            var n = it.next();
            if (i === index) {
                return n;
            }
            i++;
        }
        return null;
    };
    List.prototype.iterator = function () {
        var ln = this.first;
        var last = this.last;
        ln = ln.next;
        return {
            hasNext: function () {
                return ln !== last;
            },
            next: function () {
                var next = ln;
                ln = ln.next;
                return next.value;
            }
        };
    };
    return List;
}());
/**
 * Creates a new list
 * @return the created list
 */
function list() {
    return new List();
}
exports.default = list;
